/**
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f435_437_board.h"
#include "at32f435_437_clock.h"

#include  <os.h>
#include  <app_cfg.h>

/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/

static  OS_TCB   AppTaskStartTCB;
static  CPU_STK  AppTaskStartStk[APP_CFG_TASK_START_STK_SIZE];

static  OS_TCB   AppTask1TCB;
static  CPU_STK  AppTask1Stk[APP_CFG_TASK_1_STK_SIZE];

static  OS_TCB   AppTask2TCB;
static  CPU_STK  AppTask2Stk[APP_CFG_TASK_2_STK_SIZE];


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/

/* ----------------- APPLICATION GLOBALS -------------- */
static  OS_TCB   AppTaskStartTCB;
static  CPU_STK  AppTaskStartStk[APP_CFG_TASK_START_STK_SIZE];


/*
*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void  AppTaskStart          (void     *p_arg);
static  void  AppTaskCreate         (void);


/**
  * @brief  get system clock frequency.
  * @param  none.
  * @retval system clock frequency.
  */
uint32_t cpu_clk_freg_get(void)
{
  crm_clocks_freq_type clocks_struct;

  crm_clocks_freq_get(&clocks_struct);

  return ((uint32_t)clocks_struct.sclk_freq);
}

/**
  * @brief  this is the standard entry point for C code.  It is assumed that your code will call
  *         main() once you have performed all necessary initialization.
  * @param  none
  * @retval none
  */
int main(void)
{
  OS_ERR  err;

  /* config nvic priority group */
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);

  system_clock_config();

  /* Init uC/OS-III. */
  OSInit(&err);

  /* Create the start task */
  OSTaskCreate((OS_TCB       *)&AppTaskStartTCB,
               (CPU_CHAR     *)"App Task Start",
               (OS_TASK_PTR   )AppTaskStart,
               (void         *)0u,
               (OS_PRIO       )APP_CFG_TASK_START_PRIO,
               (CPU_STK      *)&AppTaskStartStk[0u],
               (CPU_STK_SIZE  )AppTaskStartStk[APP_CFG_TASK_START_STK_SIZE / 10u],
               (CPU_STK_SIZE  )APP_CFG_TASK_START_STK_SIZE,
               (OS_MSG_QTY    )0u,
               (OS_TICK       )0u,
               (void         *)0u,
               (OS_OPT        )(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
               (OS_ERR       *)&err);

  /* Start multitasking (i.e. give control to uC/OS-III). */
  OSStart(&err);

  (void)&err;

  return (0u);
}

/**
  * @brief  this is an example of a startup task.  As mentioned in the book's text, you MUST
  *         initialize the ticker only once multitasking has started.
  * @param  p_arg   is the argument passed to 'AppTaskStart()' by 'OSTaskCreate()'.
  * @retval none
  * @note   the first line of code is used to prevent a compiler warning because 'p_arg' is not
  *         used.  The compiler should not generate any code for this statement.
  */
static  void  AppTaskStart (void *p_arg)
{
  OS_ERR      err;

 (void)p_arg;

  at32_board_init();

  uart_print_init(115200);

	printf("This is RTOS uC/OS-III demo.\r\n");
	printf("Cortex-M4:AT32F435ZGT7.\r\n");
	printf("Gowin.\r\n");
	printf("GMD V1.2.\r\n\r\n");

  /* initialize the uC/CPU services */
  CPU_Init();

  /* init uC/OS periodic time src (SysTick). */
  OS_CPU_SysTickInitFreq(cpu_clk_freg_get());

#if OS_CFG_STAT_TASK_EN > 0u
  /* compute CPU capacity with no task running */
  OSStatTaskCPUUsageInit(&err);
#endif

#ifdef CPU_CFG_INT_DIS_MEAS_EN
  CPU_IntDisMeasMaxCurReset();
#endif
  AppTaskCreate ();
  OSTaskSuspend(&AppTaskStartTCB, &err); //Suspend start_task
}


/**
  * @brief  task 1.
  * @param  none
  * @retval none
  */
static void AppTask1(void *p_arg)
{
  OS_ERR      err;

  (void)p_arg;

  while(1)
  {
    printf("app_task_1.\r\n");
    OSTimeDly(500, OS_OPT_TIME_DLY, &err);
  }
}

/**
  * @brief  task 2.
  * @param  none
  * @retval none
  */
static void AppTask2(void *p_arg)
{
  OS_ERR      err;

  (void)p_arg;

  while(1)
  {
    printf("app_task_2.\r\n");
    OSTimeDly(1000, OS_OPT_TIME_DLY, &err);
  }
}

/**
  * @brief  create task 1/task2.
  * @param  none
  * @retval none
  */
static  void  AppTaskCreate (void)
{
  OS_ERR      err;

  /***********************************/
  OSTaskCreate((OS_TCB       *)&AppTask1TCB,
               (CPU_CHAR     *)"App Task 1",
               (OS_TASK_PTR   )AppTask1,
               (void         *)0,
               (OS_PRIO       )APP_CFG_TASK_1_PRIO,
               (CPU_STK      *)&AppTask1Stk[0],
               (CPU_STK_SIZE  )APP_CFG_TASK_1_STK_SIZE / 10,
               (CPU_STK_SIZE  )APP_CFG_TASK_1_STK_SIZE,
               (OS_MSG_QTY    )1,
               (OS_TICK       )0,
               (void         *)0,
               (OS_OPT        )(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
               (OS_ERR       *)&err);

  OSTaskCreate((OS_TCB       *)&AppTask2TCB,
               (CPU_CHAR     *)"App Task 2",
               (OS_TASK_PTR   )AppTask2,
               (void         *)0,
               (OS_PRIO       )APP_CFG_TASK_2_PRIO,
               (CPU_STK      *)&AppTask2Stk[0],
               (CPU_STK_SIZE  )APP_CFG_TASK_2_STK_SIZE / 10,
               (CPU_STK_SIZE  )APP_CFG_TASK_2_STK_SIZE,
               (OS_MSG_QTY    )2,
               (OS_TICK       )0,
               (void         *)0,
               (OS_OPT        )(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
               (OS_ERR       *)&err);
}

/**
  * @}
  */
