/*
 ******************************************************************************************
 * @file          board.c
 * @author        GowinSemicoductor
 * @device        Gowin_EMPU(GW5AS-25)
 * @brief         Driver
 ******************************************************************************************
 */

/* Includes */
#include <stdint.h>
#include <rthw.h>
#include <rtthread.h>

#include "at32f435_437_board.h"
#include "at32f435_437_clock.h"


/* Definitions */

#define _SCB_BASE       (0xE000E010UL)												//Systick
#define _SYSTICK_CTRL   (*(rt_uint32_t *)(_SCB_BASE + 0x0))
#define _SYSTICK_LOAD   (*(rt_uint32_t *)(_SCB_BASE + 0x4))
#define _SYSTICK_VAL    (*(rt_uint32_t *)(_SCB_BASE + 0x8))
#define _SYSTICK_CALIB  (*(rt_uint32_t *)(_SCB_BASE + 0xC))
#define _SYSTICK_PRI    (*(rt_uint8_t  *)(0xE000ED23UL))


// Holds the system core clock, which is the system clock 
// frequency supplied to the SysTick timer and the processor 
// core clock.
extern unsigned int system_core_clock;

static uint32_t _SysTick_Config(rt_uint32_t ticks)
{
    if ((ticks - 1) > 0xFFFFFF)
    {
        return 1;
    }
    
    _SYSTICK_LOAD = ticks - 1; 
    _SYSTICK_PRI = 0xFF;
    _SYSTICK_VAL  = 0;
    _SYSTICK_CTRL = 0x07;  
    
    return 0;
}

#if defined(RT_USING_USER_MAIN) && defined(RT_USING_HEAP)
#define RT_HEAP_SIZE 1024
static uint32_t rt_heap[RT_HEAP_SIZE];     // heap default size: 4K(1024 * 4)
RT_WEAK void *rt_heap_begin_get(void)
{
  return rt_heap;
}

RT_WEAK void *rt_heap_end_get(void)
{
  return rt_heap + RT_HEAP_SIZE;
}
#endif

/**
 * This function will initial your board.
 */
void rt_hw_board_init()
{
  system_clock_config();
  at32_board_init();
  usart_configuration();	//GW
	
  _SysTick_Config(system_core_clock / RT_TICK_PER_SECOND);

  /* Call components board initial (use INIT_BOARD_EXPORT()) */
#ifdef RT_USING_COMPONENTS_INIT
    rt_components_board_init();
#endif

#if defined(RT_USING_USER_MAIN) && defined(RT_USING_HEAP)
    rt_system_heap_init(rt_heap_begin_get(), rt_heap_end_get());
#endif
}

//Systick interrupt handler function
void SysTick_Handler(void)
{
  /* Enter interrupt */
  rt_interrupt_enter();

  rt_tick_increase();

  /* Leave interrupt */
  rt_interrupt_leave();
}

//RT console output
void rt_hw_console_output(const char *str)
{
  rt_size_t i = 0, size = 0;
  char a = '\r';

  size = rt_strlen(str);
  for (i = 0; i < size; i++)
  {
    if (*(str + i) == '\n')
    {
			while(usart_flag_get(USART1, USART_TDBE_FLAG) == RESET);
      usart_data_transmit(USART1, (uint8_t)a);
    }
		while(usart_flag_get(USART1, USART_TDBE_FLAG) == RESET);
    usart_data_transmit(USART1, (uint8_t)(*(str + i)));
  }
}

//RT console input
char rt_hw_console_getchar(void)
{
	while(usart_flag_get(USART1, USART_RDBF_FLAG) == RESET);
  return usart_data_receive(USART1);
}
