/*
 *********************************************************************************************
 * @file      delay.c
 * @author    GowinSemicoductor
 * @device    Gowin_EMPU_M1
 * @brief     Delay.
 *********************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "delay.h"


/* Definitions ---------------------------------------------------------------*/

/* Delay macros */
#define STEP_DELAY_MS                    50


/* Delay variable */
static __IO uint32_t fac_us;
static __IO uint32_t fac_ms;


/**
  * @brief  Initialize delay function
  * @param  None
  * @retval None
  */
void delay_init(void)
{
  /* Configure systick */
	SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
	
  fac_us = SystemCoreClock / (1000000U);
  fac_ms = fac_us * (1000U);
}

/**
  * @brief  Inserts a delay time.
  * @param  nus: Specifies the delay time length, in microsecond.
  * @retval None
  */
void delay_us(uint32_t nus)
{
  uint32_t temp = 0;
	
  SysTick->LOAD = (uint32_t)(nus * fac_us);
  SysTick->VAL = 0x00;
  SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk ;
	
  do
  {
    temp = SysTick->CTRL;
  }
	while((temp & 0x01) && !(temp & (1 << 16)));

  SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;
  SysTick->VAL = 0x00;
}

/**
  * @brief  Inserts a delay time.
  * @param  nms: Specifies the delay time length, in milliseconds.
  * @retval None
  */
void delay_ms(uint16_t nms)
{
  uint32_t temp = 0;
	
  while(nms)
  {
    if(nms > STEP_DELAY_MS)
    {
      SysTick->LOAD = (uint32_t)(STEP_DELAY_MS * fac_ms);
      nms -= STEP_DELAY_MS;
    }
    else
    {
      SysTick->LOAD = (uint32_t)(nms * fac_ms);
      nms = 0;
    }
		
    SysTick->VAL = 0x00;
    SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk;
		
    do
    {
      temp = SysTick->CTRL;
    }
		while((temp & 0x01) && !(temp & (1 << 16)));

    SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;
    SysTick->VAL = 0x00;
  }
}

/**
  * @brief  Inserts a delay time.
  * @param  sec: Specifies the delay time, in seconds.
  * @retval None
  */
void delay_sec(uint16_t sec)
{
  uint16_t index;
	
  for(index = 0; index < sec; index++)
  {
    delay_ms(500);
    delay_ms(500);
  }
}
