/*
 ******************************************************************************************
 * @file          main.c
 * @author        GowinSemicoductor
 * @device        Gowin_EMPU_M1
 * @brief         Main function.
 ******************************************************************************************
 */

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include  <includes.h>
#include "uart.h"
#include "led.h"

/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/

static OS_TCB startup_task_tcb;
static OS_TCB AppTaskTCB;

static CPU_STK STARTUP_TASK_STK[APP_CFG_TASK_START_STK_SIZE];
static CPU_STK APP_TASK_STK[APP_CFG_TASK_LED2_STK_SIZE];


        OS_TMR          AppLED3Tmr;
        OS_SEM          AppLED3Sem;

        OS_TMR          AppLED4Tmr;
        OS_Q            AppLED4MsgQ;

        CPU_INT08U      AppLEDNbr;

/*
*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************
*/
static void startup_task(void *p_arg);   
static void app_task(void *p_arg);

/*
*********************************************************************************************************
*                                                main()
*
* Description : This is the standard entry point for C code.  It is assumed that your code will call
*               main() once you have performed all necessary initialization.
*
* Arguments   : none
*
* Returns     : none
*********************************************************************************************************
*/

int main(void)
{
    OS_ERR      err;
#if (CPU_CFG_NAME_ERR == DEF_ENABLED)
    CPU_ERR     cpu_err;
#endif
		
		uart_init(UART0,							//Initializes UART0
							115200,							//Baudrate
							1,									//Tx
							1,									//Rx
							0,									//Tx interrupt
							0,									//Rx interrupt
							0,									//Tx overflow interrupt
							0);									//Rx overflow interrupt
    CPU_Init();

    OSInit(&err);                                                /* Initialize "uC/OS-III, The Real-Time Kernel"          */

    OSTaskCreate((OS_TCB      *)&startup_task_tcb,               /* Create the start task                                	*/
                 (CPU_CHAR    *)"startup_task",
                 (OS_TASK_PTR  )startup_task,
                 (void        *)0,
                 (OS_PRIO      )APP_CFG_TASK_START_PRIO,
                 (CPU_STK     *)&STARTUP_TASK_STK[0],
                 (CPU_STK_SIZE )APP_CFG_TASK_START_STK_SIZE_LIMIT,
                 (CPU_STK_SIZE )APP_CFG_TASK_START_STK_SIZE,
                 (OS_MSG_QTY   )0,
                 (OS_TICK      )0,
                 (void        *)0,
                 (OS_OPT       )(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                 (OS_ERR      *)&err);

    OSStart(&err);                                              /* Start multitasking (i.e. give control to uC/OS-II)   	*/

    return (1);
}


/*
*********************************************************************************************************
*                                          STARTUP TASK
*
* Description : This is an example of a startup task.  As mentioned in the book's text, you MUST
*               initialize the ticker only once multitasking has started.
*
* Arguments   : p_arg   is the argument passed to 'AppTaskStart()' by 'OSTaskCreate()'.
*
* Returns     : none
*
* Notes       : 1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                  used.  The compiler should not generate any code for this statement.
*********************************************************************************************************
*/
static void startup_task(void *p_arg)
{
    CPU_INT32U  cpu_clk_freq;
    CPU_INT32U  cnts;

    OS_ERR      os_err;

    (void)p_arg;                                                /* Note #1                                             	*/


    BSP_Init();                                                 /* Initialize BSP functions                             */

    cpu_clk_freq = BSP_CPU_ClkFreq();                           /* Determine SysTick reference freq.                    */
    cnts         = cpu_clk_freq                                 /* Determine nbr SysTick increments                     */
                 / (CPU_INT32U)OSCfg_TickRate_Hz;

    OS_CPU_SysTickInit(cnts);                                   /* Init uC/OS periodic time src (SysTick).              */


#if OS_CFG_STAT_TASK_EN > 0u
    OSStatTaskCPUUsageInit(&os_err);                            /* Compute CPU capacity with no task running            */
#endif

		OSTaskCreate((OS_TCB *		)&AppTaskTCB,
						 (CPU_CHAR *	)"app_task",
						 (OS_TASK_PTR	)app_task,
						 (void *		)0,
						 (OS_PRIO		)APP_CFG_TASK_LED2_PRIO,
						 (CPU_STK *		)&APP_TASK_STK[0],
						 (CPU_STK_SIZE	)APP_CFG_TASK_LED2_STK_SIZE_LIMIT,
						 (CPU_STK_SIZE	)APP_CFG_TASK_LED2_STK_SIZE,
						 (OS_MSG_QTY	)0,
						 (OS_TICK		)0,
						 (void *		)0,
						 (OS_OPT		)(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
						 (OS_ERR *		)&os_err);

    while(DEF_TRUE)
		{
				/* Task body, always written as an infinite loop. */
        task_led();
			
				printf("0.startup_task.\r\n");
			
        OSTimeDlyHMSM(0, 0, 0, 500,
                      OS_OPT_TIME_HMSM_STRICT,
                      &os_err);
    }
}

static void app_task(void *p_arg)
{

    OS_ERR os_err;

    (void)p_arg;

    while(1)
    {
        printf("1.app_task.\r\n");

        OSTimeDlyHMSM(0, 0, 0, 500, OS_OPT_TIME_HMSM_STRICT, &os_err);
    }
}
