
module ae250_exp
(
  CLK,
  KEY1,         // Hardware reset
  KEY2,         // Power on reset
  UART1_TXD,
  UART1_RXD,
  LED,
  JTAG_TRST,
  JTAG_TDI,
  JTAG_TMS,
  JTAG_TCK,
  JTAG_TDO,
  SPI1_CLK,
  SPI1_CSN,
  SPI1_MISO,
  SPI1_MOSI,
  I2C1_SCL,
  I2C1_SDA,
  PWM_CH0,
  PWM_CH1,
  PWM_CH2,
  PWM_CH3
) ;

input CLK;
input KEY1;
input KEY2;

//UART1
output UART1_TXD;
input UART1_RXD;

//LED
inout [3:0] LED;

//JTAG
inout JTAG_TRST;
inout JTAG_TDI;
inout JTAG_TMS;
inout JTAG_TCK;
inout JTAG_TDO;

//SPI1
inout SPI1_CLK;
inout SPI1_CSN;
inout SPI1_MISO;
inout SPI1_MOSI;

//I2C1
inout I2C1_SCL;
inout I2C1_SDA;

//PIT1
output PWM_CH0;
output PWM_CH1;
output PWM_CH2;
output PWM_CH3;

wire hwRstn = KEY1;
wire por = KEY2;
wire sysClk;
wire rtcClk;

//RiscV_AE250_Top
RiscV_AE250_Top u_RiscV_AE250_Top
(
  .X_om(),
  .X_pd_pwr_off(),
  .X_aopd_por_b(por),
  .X_mpd_pwr_off(),
  .X_osclin(rtcClk),
  .X_osclio(),
  .X_rtc_wakeup(),
  .X_wakeup_in(1'b0),
  .X_tck(JTAG_TCK),
  .X_oschin(sysClk),
  .X_oschio(),
  .X_hw_rstn(hwRstn),
  .X_por_b(por),
  .X_tms(JTAG_TMS),
  .X_tdi(JTAG_TDI), 
  .X_tdo(JTAG_TDO),
  .X_trst(JTAG_TRST),
  .X_spi1_clk(SPI1_CLK),
  .X_spi1_csn(SPI1_CSN),
  .X_spi1_miso(SPI1_MISO),
  .X_spi1_mosi(SPI1_MOSI),
  .X_i2c1_scl(I2C1_SCL),
  .X_i2c1_sda(I2C1_SDA),
  .X_uart1_ctsn(),
  .X_uart1_rtsn(),
  .X_uart1_dcdn(),
  .X_uart1_dsrn(),
  .X_uart1_dtrn(),
  .X_uart1_out1n(),
  .X_uart1_out2n(),
  .X_uart1_rin(),
  .X_uart1_rxd(UART1_RXD),
  .X_uart1_txd(UART1_TXD),
  .X_pit1_pwm_ch0(PWM_CH0),
  .X_pit1_pwm_ch1(PWM_CH1),
  .X_pit1_pwm_ch2(PWM_CH2),
  .X_pit1_pwm_ch3(PWM_CH3),
  .X_gpio(LED),
  .gpio_pullup(),
  .gpio_pulldown(),
  .ext_int_src()
); 

//rPLL
Gowin_rPLL u_Gowin_rPLL (
  .clkout(sysClk),  //40MHz
  .clkoutd(rtcClk),
  .clkin(CLK)
);

endmodule