/*
 * ******************************************************************************************
 * File		: interrupt.c
 * Author 	: GowinSemicoductor
 * Chip		: AE250_N25
 * Function	: Interrupt handler
 * ******************************************************************************************
 * */

// Include ----------------------------------------------------------------------------------
#include <stdio.h>
#include "platform.h"


// Definition -------------------------------------------------------------------------------
typedef void (*isr_func)(void);

static void default_irq_handler(void)
{
	printf("Default interrupt handler\n");
}

//Interrupt handler
void wdt_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void rtc_period_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void rtc_alarm_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void pit_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void spi1_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void spi2_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void i2c_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void gpio_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void uart1_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void uart2_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void dmac_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void bmc_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void swint_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void sdc_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void mac_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void standby_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));
void wakeup_irq_handler(void) __attribute__((weak, alias("default_irq_handler")));

//Interrupt handler entry
const isr_func irq_handler[] =
{
	wdt_irq_handler,         //0
	rtc_period_irq_handler,  //1
	rtc_alarm_irq_handler,   //2
	pit_irq_handler,         //3
	spi1_irq_handler,        //4
	spi2_irq_handler,        //5
	i2c_irq_handler,         //6
	gpio_irq_handler,        //7
	uart1_irq_handler,       //8
	uart2_irq_handler,       //9
	dmac_irq_handler,        //10
	bmc_irq_handler,         //11
	swint_irq_handler,       //12
	default_irq_handler,     //13
	default_irq_handler,     //14
	default_irq_handler,     //15
	default_irq_handler,     //16
	default_irq_handler,     //17
	sdc_irq_handler,         //18
	mac_irq_handler,         //19
	default_irq_handler,     //20
	default_irq_handler,     //21
	default_irq_handler,     //22
	default_irq_handler,     //23
	default_irq_handler,     //24
	default_irq_handler,     //25
	standby_irq_handler,     //26
	wakeup_irq_handler,      //27
	default_irq_handler,     //28
	default_irq_handler,     //29
	default_irq_handler,     //30
	default_irq_handler      //31
};

void interrupt_handler(unsigned int irq_source)
{
	/* Enable interrupts in general to allow nested */
	set_csr(NDS_MSTATUS, MSTATUS_MIE);

	/* Do interrupt handler */
	irq_handler[irq_source]();

	__nds__plic_complete_interrupt(irq_source);
}
