/*
 * ******************************************************************************************
 * File		: uart.c
 * Author 	: GowinSemicoductor
 * Chip		: AE250_N25
 * Function	: UART1 send and receive
 * ******************************************************************************************
 * */

// Include ----------------------------------------------------------------------------------
#include "platform.h"
#include "uart.h"


// Definition -------------------------------------------------------------------------------
#define BAUD_RATE(n)            (UCLKFREQ / (n) / 16)

//Initializes uart1
void uart1_init(unsigned int baudrate)
{
	/* Set DLAB to 1 */
	AE250_UART1->LCR |= 0x80;

	/* Set DLL for baudrate */
	AE250_UART1->DLL = (BAUD_RATE(baudrate) >> 0) & 0xff;
	AE250_UART1->DLM = (BAUD_RATE(baudrate) >> 8) & 0xff;

	/* LCR: Length 8, no parity, 1 stop bit. */
	AE250_UART1->LCR = 0x03;

	/* FCR: Enable FIFO, reset TX and RX. */
	AE250_UART1->FCR = 0x07;
}

//Send data
void uart1_putc(int c)
{
	#define SERIAL_LSR_THRE         0x20

	while ((AE250_UART1->LSR & SERIAL_LSR_THRE) == 0);

	AE250_UART1->THR = c;
}

//Receive data
void uart1_getc(char *c)
{
	#define LSR_DR 0x01

	while(((AE250_UART1->LSR) & 0x01)  == 0);

	*c = AE250_UART1->RBR;
}

//Send byte
int outbyte(int c)
{
	uart1_putc(c);

	if (c =='\n')
	{
		uart1_putc('\r');
	}

	return c;
}
