/*
 * ******************************************************************************************
 * File		: load.c
 * Author 	: GowinSemicoductor
 * Chip		: AE250_N25
 * Function	: Load
 * ******************************************************************************************
 * */

// Include ----------------------------------------------------------------------------------
#include "config.h"


// Definition -------------------------------------------------------------------------------
#ifdef CFG_BURN

#ifdef CFG_GCOV
/* No GCOV available */
#pragma GCC optimize "no-test-coverage"
#pragma GCC optimize "no-profile-arcs"
#endif

extern unsigned long __flash_start, __text_start, __text_lmastart, __text_lmaend, __data_start, __data_lmastart, __data_lmaend;
extern unsigned long __loader_start, __loader_lmaend, __loader_lmastart;


/* The entry function when boot, executing on ROM/FLASH. */
void __attribute__((no_prologue, optimize("O0"), section(".bootloader"))) boot_loader(void)
{
	asm (".no_ex9_begin");
	register unsigned long *src_ptr, *dst_ptr;
	register unsigned long i, size;
	register void (*ldr_entry)(void) = (void *)&__loader_start;

	/* Initial $GP incase used by compiler */
	asm ("la gp, __global_pointer$");

	/* Copy loader() code to VMA area (On memory) */
	size = ((unsigned long)&__loader_lmaend - (unsigned long)&__loader_lmastart + (sizeof(long) - 1)) / sizeof(long);

	src_ptr = (unsigned long *)((unsigned long)&__flash_start + (unsigned long)&__loader_lmastart);
	dst_ptr = (unsigned long *)&__loader_start;

	for (i = 0; i < size; i++)
	{
		*dst_ptr++ = *src_ptr++;
	}

	/*
	 * Call the real program loader which executes on memory instead
	 * of flash for batter coping performance.
	 */
	ldr_entry();

	asm (".no_ex9_end");
}

void __attribute__((no_prologue, optimize("O0"), section(".loader"))) loader(void)
{
	asm (".no_ex9_begin");
	register unsigned long *src_ptr, *dst_ptr;
	register unsigned long i, size;
	register void (*entry)(void) = (void *)&__text_start;

	/* Copy TEXT section to VMA area (On memory) */
	size = ((unsigned long)&__text_lmaend - (unsigned long)&__text_lmastart + (sizeof(long) - 1)) / sizeof(long);

	src_ptr = (unsigned long *)((unsigned long)&__flash_start + (unsigned long)&__text_lmastart);
	dst_ptr = (unsigned long *)&__text_start;

	for (i = 0; i < size; i++)
	{
		*dst_ptr++ = *src_ptr++;
	}

	/* Copy DATA section to VMA area (On memory) */
	size = ((unsigned long)&__data_lmaend - (unsigned long)&__data_lmastart + (sizeof(long) - 1)) / sizeof(long);

	src_ptr = (unsigned long *)((unsigned long)&__flash_start + (unsigned long)&__data_lmastart);
	dst_ptr = (unsigned long *)&__data_start;

	for (i = 0; i < size; i++)
	{
		*dst_ptr++ = *src_ptr++;
	}

	entry();
}

asm (".no_ex9_end");

#endif	// CFG_BURN
