/*
 * ******************************************************************************************
 * File		: start.S
 * Author 	: GowinSemicoductor
 * Chip		: AE250_N25
 * Function	: Start up
 * ******************************************************************************************
 * */

// Include ----------------------------------------------------------------------------------
#include "config.h"
#include "core_v5.h"


// Definition -------------------------------------------------------------------------------

	.section .init

	.global reset_vector

reset_vector:
	/* Decide whether this is an NMI or cold reset */
	csrr t0, mcause
	bnez t0, nmi_handler

	.global _start
	.type _start,@function

_start:
	/* Initialize global pointer */
	la gp, __global_pointer$

	/* Initialize stack pointer */
	la t0, _stack
	mv sp, t0

#ifdef __nds_ex9
	/* Initialize EX9 table */
	la t0, _ITB_BASE_
	csrw uitb, t0
#endif

	/* Initial machine trap-vector Base */
	la t0, trap_entry
	csrw mtvec, t0

	/* Do system low level setup. It must be a leaf function */
	call __platform_init

#ifndef CFG_BURN
	/* Load data section */
	la a0, __data_lmastart
	la a1, __data_start
	la a2, _edata
	bgeu a1, a2, 2f
1:
	lw t0, (a0)
	sw t0, (a1)
	addi a0, a0, 4
	addi a1, a1, 4
	bltu a1, a2, 1b
2:
#endif

	/* Clear bss section */
	la a0, _edata
	la a1, _end
	bgeu a0, a1, 2f
1:
	sw zero, (a0)
	addi a0, a0, 4
	bltu a0, a1, 1b
2:

	/* System reset handler */
	call reset_handler

	/* Infinite loop, if returned accidently */
1:	j 1b

	.weak __platform_init
__platform_init:
	ret

	.section .text
	.align 2

trap_entry:
	/* Check for Machine timer interrupt */
	/* FreeRTOS */
	STORE t0, -1*REGBYTES(sp)
	csrr t0, mcause
	bgez t0, 1f
	andi t0, t0, 255
	addi t0, t0, -7
	bnez t0, 1f
	LOAD t0, -1*REGBYTES(sp)
	j FreeRTOS_MTIMER_Handler

1:
	LOAD t0, -1*REGBYTES(sp)
	addi sp, sp, -32*REGBYTES

	/* Save caller registers */
	STORE x1, 0*REGBYTES(sp)
	STORE x4, 1*REGBYTES(sp)
	STORE x5, 2*REGBYTES(sp)
	STORE x6, 3*REGBYTES(sp)
	STORE x7, 4*REGBYTES(sp)
	STORE x10, 5*REGBYTES(sp)
	STORE x11, 6*REGBYTES(sp)
	STORE x12, 7*REGBYTES(sp)
	STORE x13, 8*REGBYTES(sp)
	STORE x14, 9*REGBYTES(sp)
	STORE x15, 10*REGBYTES(sp)
#ifndef __riscv_32e
	STORE x16, 11*REGBYTES(sp)
	STORE x17, 12*REGBYTES(sp)
	STORE x28, 13*REGBYTES(sp)
	STORE x29, 14*REGBYTES(sp)
	STORE x30, 15*REGBYTES(sp)
	STORE x31, 16*REGBYTES(sp)
#endif

	/* Save mstatus and mepc */
	csrr t0, mepc
	csrr t1, mstatus
	csrr t2, mxstatus

	STORE t0, 17*REGBYTES(sp)
	STORE t1, 18*REGBYTES(sp)
	STORE t2, 19*REGBYTES(sp)

	/* Call trap handler */
	csrr a0, mcause
	mv a1, sp
	call trap_handler

	/* Disable interrupt */
	csrrci a5, mstatus, MSTATUS_MIE

	/* Restore mstatus and mepc */
	LOAD t0, 17*REGBYTES(sp)
	LOAD t1, 18*REGBYTES(sp)
	LOAD t2, 19*REGBYTES(sp)

	csrw mepc, t0
	csrw mstatus, t1
	csrw mxstatus, t2

	/* Restore caller registers */
	LOAD x1, 0*REGBYTES(sp)
	LOAD x4, 1*REGBYTES(sp)
	LOAD x5, 2*REGBYTES(sp)
	LOAD x6, 3*REGBYTES(sp)
	LOAD x7, 4*REGBYTES(sp)
	LOAD x10, 5*REGBYTES(sp)
	LOAD x11, 6*REGBYTES(sp)
	LOAD x12, 7*REGBYTES(sp)
	LOAD x13, 8*REGBYTES(sp)
	LOAD x14, 9*REGBYTES(sp)
	LOAD x15, 10*REGBYTES(sp)
#ifndef __riscv_32e
	LOAD x16, 11*REGBYTES(sp)
	LOAD x17, 12*REGBYTES(sp)
	LOAD x28, 13*REGBYTES(sp)
	LOAD x29, 14*REGBYTES(sp)
	LOAD x30, 15*REGBYTES(sp)
	LOAD x31, 16*REGBYTES(sp)
#endif

	addi sp, sp, 32*REGBYTES
	mret

	.weak trap_handler
trap_handler:
1:	j 1b

	.weak nmi_handler
nmi_handler:
1:	j 1b
