/*
 * ******************************************************************************************
 * File		: trap.c
 * Author 	: GowinSemicoductor
 * Chip		: AE250_N25
 * Function	: Trap handler
 * ******************************************************************************************
 * */

// Include ----------------------------------------------------------------------------------
#include <stdio.h>
#include "platform.h"


// Definition -------------------------------------------------------------------------------
typedef struct
{
	union
	{
		struct
		{
			long x1;
			long x4;
			long x5;
			long x6;
			long x7;
			long x10;
			long x11;
			long x12;
			long x13;
			long x14;
			long x15;
			long x16;
			long x17;
			long x28;
			long x29;
			long x30;
			long x31;
		};
		long caller_regs[17];
	};
	long mepc;
	long mstatus;
	long mxstatus;
} SAVED_CONTEXT;

extern void interrupt_handler(unsigned int irq_source);

void mtime_handler(void) __attribute__((weak, alias("default_mtime_handler")));
void mswi_handler(void) __attribute__((weak, alias("default_mswi_handler")));
void syscall_handler(long, long, long, long, long) __attribute__((weak, alias("default_syscall_handler")));
long except_handler(long, long, long *) __attribute__((weak, alias("default_except_handler")));

static void default_mtime_handler(void)
{
	clear_csr(NDS_MIE, MIP_MTIP);
}

static void default_mswi_handler(void)
{
	clear_csr(NDS_MIE, MIP_MSIP);
}

static void default_syscall_handler(long n, long a0, long a1, long a2, long a3)
{
	printf("syscall #%ld (a0:0x%lx,a1:0x%lx, a2:0x%lx, a3:0x%lx)\n", n, a0, a1, a2, a3);
}

static long default_except_handler(long cause, long epc, long *reg)
{
	/* Unhandled Trap */
	printf("Unhandled Trap : mcause = 0x%x, mepc = 0x%x\n", (unsigned int)cause, (unsigned int)epc);

	return epc;
}

void trap_handler(unsigned long mcause, SAVED_CONTEXT *context)
{
	/* Do your trap handling */
	if ((mcause & MCAUSE_INT) && ((mcause & MCAUSE_CAUSE) == IRQ_M_EXT))
	{
		/* Machine-level interrupt from PLIC */
		interrupt_handler(__nds__plic_claim_interrupt());
	}
	else if ((mcause & MCAUSE_INT) && ((mcause & MCAUSE_CAUSE) == IRQ_M_TIMER))
	{
		/* Machine timer interrupt */
		mtime_handler();
	}
	else if ((mcause & MCAUSE_INT) && ((mcause & MCAUSE_CAUSE) == IRQ_M_SOFT))
	{
		/* Machine SWI interrupt */
		mswi_handler();
	}
	else if (!(mcause & MCAUSE_INT) && ((mcause & MCAUSE_CAUSE) == TRAP_M_ECALL))
	{
		/* Syscal call */
		syscall_handler(context->x17, context->x10, context->x11, context->x12, context->x13);
		context->mepc += 4;
	}
	else
	{
		/* Unhandled Trap */
		context->mepc = except_handler(mcause, context->mepc, context->caller_regs);
	}
}
