/*
 * ******************************************************************************************
 * File		: main.c
 * Author 	: GowinSemicoductor
 * Chip		: AE250_N25
 * Function	: Main function
 * ******************************************************************************************
 * */

// Include ----------------------------------------------------------------------------------
#include <stdio.h>
#include "FreeRTOS.h"
#include "task.h"
#include "Driver_GPIO.h"


// Definition -------------------------------------------------------------------------------
#define SW_STR_NAME							"FreeRTOS"
#define SW_STR_EDITION						"V9.0.0"
#define SW_STR_AUTHOR						"Gowin"


#define LOGO_PRINT_ON

#ifdef LOGO_PRINT_ON
#include "logo.h"
#endif	//LOGO_PRINT_ON

#define LED0_PIN				0
#define LED1_PIN				1

extern NDS_DRIVER_GPIO Driver_GPIO;

//Task 0
#define TASK0_TASK_PRIO			1
#define TASK0_STK_SIZE 			48
TaskHandle_t TASK0_Handler;
static void task0(void *pvParameters);

// Task 1
#define TASK1_TASK_PRIO			2
#define TASK1_STK_SIZE 			48
TaskHandle_t TASK1_Handler;
static void task1(void *pvParameters);

volatile int led0_flag = 0;
volatile int led1_flag = 0;


static void sw_edition_print(void)
{
	printf("\r\n");
	printf(starts_str);
	printf("Name:     "SW_STR_NAME"\r\n"
           "Edition:  "SW_STR_EDITION"\r\n"
		   "Compiled: "__DATE__", "__TIME__"\r\n"
			"Author:   "SW_STR_AUTHOR"\r\n");
	printf(starts_str);
	printf("\r\n");

#ifdef LOGO_PRINT_ON
	printf("\r\n");
	printf(starts_str);
	printf(logo_str);
	printf(starts_str);
	printf("\r\n");
#endif	//LOGO_PRINT_ON
}

static void prvSetupHardware( void )
{
	/* Ensure no interrupts execute while the scheduler is in an inconsistent
	state.  Interrupts are automatically enabled when the scheduler is
	started. */
	portDISABLE_INTERRUPTS();
}

void vAssertCalled(const char * pcFile, unsigned long ulLine)
{
	volatile unsigned long ul = 0;

	(void) pcFile;
	(void) ulLine;

	printf("ASSERT! Line %d, file %s\r\n", ( int )ulLine, pcFile);

	taskENTER_CRITICAL();
	{
		/* Set ul to a non-zero value using the debugger to step out of this
		function. */
		while (ul == 0)
		{
			portNOP();
		}
	}
	taskEXIT_CRITICAL();
}

//GPIO callback event
void gpio_callback(uint32_t event)
{
	switch (event)
	{
		// Don't have GPIO pins as input interrupt event
		default:
			break;
	}
}

//Initializes GPIO
static void gpio_init(void)
{
	NDS_DRIVER_GPIO *GPIO_Dri = &Driver_GPIO;

	GPIO_Dri->Initialize(gpio_callback);	//Initializes

	GPIO_Dri->SetDir((1 << LED0_PIN) | (1 << LED1_PIN), NDS_GPIO_DIR_OUTPUT);	//Set output direction

	GPIO_Dri->PinWrite(LED0_PIN, 1);	//Set LED1 off
	GPIO_Dri->PinWrite(LED1_PIN, 1);	//Set LED2 off
}

//Task thread 0
static void task0(void *pvParameters)
{
	NDS_DRIVER_GPIO *GPIO_Dri = &Driver_GPIO;

    while (1)
	{
		if (0 == led0_flag)
		{
			GPIO_Dri->PinWrite(LED0_PIN, 0);	//LED1 on
		}
		else
		{
			GPIO_Dri->PinWrite(LED0_PIN, 1);	//LED1 off
		}

		led0_flag = !led0_flag;

		printf("* 0.task0 *\r\n");

		vTaskDelay(pdMS_TO_TICKS(250));
	}
}

//Task thread 1
static void task1(void *pvParameters)
{
	NDS_DRIVER_GPIO *GPIO_Dri = &Driver_GPIO;

    while (1)
	{
		if (0 == led1_flag)
		{
			GPIO_Dri->PinWrite(LED1_PIN, 0);	//LED2 on
		}
		else
		{
			GPIO_Dri->PinWrite(LED1_PIN, 1);	//LED2 off
		}

		led1_flag = !led1_flag;

		printf("* 1.task1 *\r\n\r\n");

		vTaskDelay(pdMS_TO_TICKS(1000));
	}
}

// Main entry
int main(void)
{
	prvSetupHardware();

	gpio_init();	//Initializes GPIO
	sw_edition_print();	//Print edition

	//Task threads
	taskENTER_CRITICAL();
	{
		//Thread 0
		xTaskCreate((TaskFunction_t )task0,
					(const char *   )"task0",
					(const uint16_t )TASK0_STK_SIZE,
					(void * const   )NULL,
					(UBaseType_t    )TASK0_TASK_PRIO,
					(TaskHandle_t *  )&TASK0_Handler);

		//Thread 1
		xTaskCreate((TaskFunction_t )task1,
					(const char *   )"task1",
					(const uint16_t )TASK1_STK_SIZE,
					(void * const   )NULL,
					(UBaseType_t    )TASK1_TASK_PRIO,
					(TaskHandle_t *  )&TASK1_Handler);
	}
    taskEXIT_CRITICAL();

	vTaskStartScheduler();

	while (1);

	return 0;
}
