/*
 * ******************************************************************************************
 * File		: start.S
 * Author 	: GowinSemicoductor
 * Chip		: AE250_N25
 * Function	: Start up for uC/OS-III revision
 * ******************************************************************************************
 * */

// Include ----------------------------------------------------------------------------------
#include "config.h"
#include "core_v5.h"


// Definition -------------------------------------------------------------------------------

	.section .init

	.global reset_vector

reset_vector:
	/* Decide whether this is an NMI or cold reset */
	csrr t0, mcause
	bnez t0, nmi_handler

	.global _start
	.type _start,@function

_start:
	/* Initialize global pointer */
	la gp, __global_pointer$

	/* Initialize stack pointer */
	la t0, _stack
	mv sp, t0

#ifdef __nds_ex9
	/* Initialize EX9 table */
	la t0, _ITB_BASE_
	csrw uitb, t0
#endif

	/* Initial machine trap-vector Base */
	la t0, trap_entry
	csrw mtvec, t0

	/* Do system low level setup. It must be a leaf function */
	call __platform_init

#ifndef CFG_BURN
	/* Load data section */
	la a0, __data_lmastart
	la a1, __data_start
	la a2, _edata
	bgeu a1, a2, 2f
1:
	lw t0, (a0)
	sw t0, (a1)
	addi a0, a0, 4
	addi a1, a1, 4
	bltu a1, a2, 1b
2:
#endif

	/* Clear bss section */
	la a0, _edata
	la a1, _end
	bgeu a0, a1, 2f
1:
	sw zero, (a0)
	addi a0, a0, 4
	bltu a0, a1, 1b
2:

	/* System reset handler */
	call reset_handler

	/* Infinite loop, if returned accidently */
1:	j 1b

	.weak __platform_init
__platform_init:
	ret

	.section .text
	.align 2

/* Specific uC/OS-III*/
trap_entry:
	addi sp, sp, -32*REGBYTES

	/* Save caller registers */
	STORE x1, 0 * REGBYTES(sp)
	STORE x2, 1 * REGBYTES(sp)
	STORE x3, 2 * REGBYTES(sp)
	STORE x4, 3 * REGBYTES(sp)
	STORE x5, 4 * REGBYTES(sp)
	STORE x6, 5 * REGBYTES(sp)
	STORE x7, 6 * REGBYTES(sp)
	STORE x8, 7 * REGBYTES(sp)
	STORE x9, 8 * REGBYTES(sp)
	STORE x10, 9 * REGBYTES(sp)
	STORE x11, 10 * REGBYTES(sp)
	STORE x12, 11 * REGBYTES(sp)
	STORE x13, 12 * REGBYTES(sp)
	STORE x14, 13 * REGBYTES(sp)
	STORE x15, 14 * REGBYTES(sp)
	STORE x16, 15 * REGBYTES(sp)
	STORE x17, 16 * REGBYTES(sp)
	STORE x18, 17 * REGBYTES(sp)
	STORE x19, 18 * REGBYTES(sp)
	STORE x20, 19 * REGBYTES(sp)
	STORE x21, 20 * REGBYTES(sp)
	STORE x22, 21 * REGBYTES(sp)
	STORE x23, 22 * REGBYTES(sp)
	STORE x24, 23 * REGBYTES(sp)
	STORE x25, 24 * REGBYTES(sp)
	STORE x26, 25 * REGBYTES(sp)
	STORE x27, 26 * REGBYTES(sp)
	STORE x28, 27 * REGBYTES(sp)
	STORE x29, 28 * REGBYTES(sp)
	STORE x30, 29 * REGBYTES(sp)
	STORE x31, 30 * REGBYTES(sp)

	csrr t0, mepc
	STORE t0, 31 * REGBYTES(sp)

	csrr a0, mcause
	mv a1, sp
	mv a2, sp
	jal trap_handler

	LOAD t0, 31 * REGBYTES(sp)
	csrw mepc, t0

	/* Remain in M-mode after mret */
	li t0, MSTATUS_MPP
	csrs mstatus, t0

	LOAD x1, 0 * REGBYTES(sp)
	LOAD x2, 1 * REGBYTES(sp)
	LOAD x3, 2 * REGBYTES(sp)
	LOAD x4, 3 * REGBYTES(sp)
	LOAD x5, 4 * REGBYTES(sp)
	LOAD x6, 5 * REGBYTES(sp)
	LOAD x7, 6 * REGBYTES(sp)
	LOAD x8, 7 * REGBYTES(sp)
	LOAD x9, 8 * REGBYTES(sp)
	LOAD x10, 9 * REGBYTES(sp)
	LOAD x11, 10 * REGBYTES(sp)
	LOAD x12, 11 * REGBYTES(sp)
	LOAD x13, 12 * REGBYTES(sp)
	LOAD x14, 13 * REGBYTES(sp)
	LOAD x15, 14 * REGBYTES(sp)
	LOAD x16, 15 * REGBYTES(sp)
	LOAD x17, 16 * REGBYTES(sp)
	LOAD x18, 17 * REGBYTES(sp)
	LOAD x19, 18 * REGBYTES(sp)
	LOAD x20, 19 * REGBYTES(sp)
	LOAD x21, 20 * REGBYTES(sp)
	LOAD x22, 21 * REGBYTES(sp)
	LOAD x23, 22 * REGBYTES(sp)
	LOAD x24, 23 * REGBYTES(sp)
	LOAD x25, 24 * REGBYTES(sp)
	LOAD x26, 25 * REGBYTES(sp)
	LOAD x27, 26 * REGBYTES(sp)
	LOAD x28, 27 * REGBYTES(sp)
	LOAD x29, 28 * REGBYTES(sp)
	LOAD x30, 29 * REGBYTES(sp)
	LOAD x31, 30 * REGBYTES(sp)

	addi sp, sp, 32*REGBYTES
	mret

	.weak trap_handler
trap_handler:
1:	j 1b

	.weak nmi_handler
nmi_handler:
1:	j 1b
