/*
 * ******************************************************************************************
 * File		: main.c
 * Author 	: GowinSemicoductor
 * Chip		: AE250_N25
 * Function	: Main function
 * ******************************************************************************************
 * */
 
// Include ----------------------------------------------------------------------------------
#include <stdio.h>
#include "includes.h"
#include "Driver_GPIO.h"
#include "platform.h"


// Definition -------------------------------------------------------------------------------
#define SW_STR_NAME							"UC/OS-III"
#define SW_STR_EDITION						"V3.06.02"
#define SW_STR_AUTHOR						"Gowin"


#define LOGO_PRINT_ON

#ifdef LOGO_PRINT_ON
#include "logo.h"
#endif	//LOGO_PRINT_ON


#define PLIC_SW_SOURCE			1

#define TICK_CLOCK_HZ			(MTIMERFREQ)
#define MTIME_BASE_ADDR			(0xE6000000)


#define LED0_PIN				0
#define LED1_PIN				1

extern NDS_DRIVER_GPIO Driver_GPIO;	//GPIO

//Task 0: startup task thread
#define STARTUP_TASK_PRIO		3
#define STARTUP_STK_SIZE 		80
OS_TCB startup_task_tcb;
CPU_STK STARTUP_TASK_STK[STARTUP_STK_SIZE];
static void startup_task(void *p_arg);

//Task 1: application task thread
#define APP_TASK_PRIO			(STARTUP_TASK_PRIO + 1)
#define APP_STK_SIZE 			80
OS_TCB app_task_tcb;
CPU_STK APP_TASK_STK[APP_STK_SIZE];
static void app_task(void *p_arg);


volatile int led0_flag = 0;
volatile int led1_flag = 0;


extern void SysTick_Handler(void);
extern void Software_IRQHandler();


//Print edition
static void sw_edition_print(void)
{
	printf("\r\n");
	printf(starts_str);
	printf("Name:     "SW_STR_NAME"\r\n"
           "Edition:  "SW_STR_EDITION"\r\n"
		   "Compiled: "__DATE__", "__TIME__"\r\n"
			"Author:   "SW_STR_AUTHOR"\r\n");
	printf(starts_str);
	printf("\r\n");

#ifdef LOGO_PRINT_ON
	printf("\r\n");
	printf(starts_str);
	printf(logo_str);
	printf(starts_str);
	printf("\r\n");
#endif	//LOGO_PRINT_ON
}

//GPIO callback event
void gpio_callback(uint32_t event)
{
	switch (event)
	{
		// Don't have GPIO pins as input interrupt event
		default:
			break;
	}
}

//Initializes GPIO
static void gpio_init(void)
{
	NDS_DRIVER_GPIO *GPIO_Dri = &Driver_GPIO;

	GPIO_Dri->Initialize(gpio_callback);	//Initializes

	GPIO_Dri->SetDir((1 << LED0_PIN) | (1 << LED1_PIN), NDS_GPIO_DIR_OUTPUT);	//Set output direction

	GPIO_Dri->PinWrite(LED0_PIN, 1);	//LED1 off
	GPIO_Dri->PinWrite(LED1_PIN, 1);	//LED2 off
}

//Set machine timer
static void mtimer_next_time_set(void)
{
	volatile uint64_t *mtime = (volatile uint64_t * const)(MTIME_BASE_ADDR);
	volatile uint64_t *timecmp = (volatile uint64_t * const)(MTIME_BASE_ADDR + 8);

	*timecmp = *mtime + (TICK_CLOCK_HZ / OS_CFG_TICK_RATE_HZ);
}

//Initializes swi interrupt
void timer_swi_init(void)
{
	mtimer_next_time_set();

	__nds__plic_sw_enable_interrupt(PLIC_SW_SOURCE);

	set_csr(NDS_MIE, MIP_MTIP);
}

unsigned int plic_sw_claim_interrupt_func(void)
{
  unsigned int hart_id = __nds__mfsr(NDS_MHARTID);

  volatile unsigned int *claim_addr = (volatile unsigned int *)(NDS_PLIC_SW_BASE +
                                                                PLIC_CLAIM_OFFSET +
                                                                (hart_id << PLIC_CLAIM_SHIFT_PER_TARGET));

  return *claim_addr;
}

void plic_sw_complete_interrupt_func(void)
{
  unsigned int hart_id = __nds__mfsr(NDS_MHARTID);

  volatile unsigned int *claim_addr = (volatile unsigned int *)(NDS_PLIC_SW_BASE +
                                                                PLIC_CLAIM_OFFSET +
                                                                (hart_id << PLIC_CLAIM_SHIFT_PER_TARGET));

  *claim_addr = PLIC_SW_SOURCE;
}

//Machine timer interrupt handler
void mtime_handler(void)
{
	clear_csr(NDS_MIE, MIP_MTIP);

	mtimer_next_time_set();
    SysTick_Handler();

    set_csr(NDS_MIE, MIP_MTIP);
}

//Machine swi interrupt handler
void mswi_handler(void)
{
	Software_IRQHandler();
}

//Initializes rtos interrupt
static void os_interrupt_init(void)
{
	__nds__plic_sw_set_priority(1, 1);
	__nds__plic_sw_disable_interrupt(1);

	clear_csr(NDS_MSTATUS, MSTATUS_MIE);
	clear_csr(NDS_MIE, MIP_MTIP);
}

//Task thread 1: application task
static void app_task(void *p_arg)
{
	OS_ERR os_err;

	(void)p_arg;

	NDS_DRIVER_GPIO *GPIO_Dri = &Driver_GPIO;

	while (1)
	{
		if (0 == led1_flag)
		{
			GPIO_Dri->PinWrite(LED1_PIN, 0);	//LED2 on
		}
		else
		{
			GPIO_Dri->PinWrite(LED1_PIN, 1);	//LED2 off
		}

		led1_flag = !led1_flag;
		printf("* 1.app_task *\r\n\r\n");
		OSTimeDlyHMSM(0, 0, 0, 1000, OS_OPT_TIME_HMSM_STRICT, &os_err);
	}
}

//Task thread 1: startup task
static void startup_task(void *p_arg)
{
	OS_ERR os_err;
	CPU_SR_ALLOC();

	(void)p_arg;

	NDS_DRIVER_GPIO *GPIO_Dri = &Driver_GPIO;

	CPU_CRITICAL_ENTER();
	{
		OSTaskCreate((OS_TCB *		)&app_task_tcb,
					 (CPU_CHAR *	)"app_task",
					 (OS_TASK_PTR	)app_task,
					 (void *		)0,
					 (OS_PRIO		)APP_TASK_PRIO,
					 (CPU_STK *		)&APP_TASK_STK[0],
					 (CPU_STK_SIZE	)(APP_STK_SIZE / 10),
					 (CPU_STK_SIZE	)APP_STK_SIZE,
					 (OS_MSG_QTY	)0,
					 (OS_TICK		)0,
					 (void *		)0,
					 (OS_OPT		)(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
					 (OS_ERR *		)&os_err);

		if (OS_ERR_NONE != os_err)
		{
			while (1);
		}
	}
	CPU_CRITICAL_EXIT();

    while (1)
	{
		if (0 == led0_flag)
		{
			GPIO_Dri->PinWrite(LED0_PIN, 0);	//LED1 on
		}
		else
		{
			GPIO_Dri->PinWrite(LED0_PIN, 1);	//LED1 off
		}

		led0_flag = !led0_flag;
		printf("* 0.startup_task *\r\n");
		OSTimeDlyHMSM(0, 0, 0, 250, OS_OPT_TIME_HMSM_STRICT, &os_err);
	}
}

//Main entry
int main(void)
{
	OS_ERR os_err;
	CPU_SR_ALLOC();

	os_interrupt_init();
    CPU_Init();
    gpio_init();

    sw_edition_print();

    OSInit(&os_err);
    if (OS_ERR_NONE != os_err)
	{
        while (1);
    }

	CPU_CRITICAL_ENTER();
	{
		OSTaskCreate((OS_TCB *		)&startup_task_tcb,
					 (CPU_CHAR *	)"startup_task",
					 (OS_TASK_PTR	)startup_task,
					 (void *		)0,
					 (OS_PRIO		)STARTUP_TASK_PRIO,
					 (CPU_STK *		)&STARTUP_TASK_STK[0],
					 (CPU_STK_SIZE	)(STARTUP_STK_SIZE / 10),
					 (CPU_STK_SIZE	)STARTUP_STK_SIZE,
					 (OS_MSG_QTY	)0,
					 (OS_TICK		)0,
					 (void *		)0,
					 (OS_OPT		)(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
					 (OS_ERR *		)&os_err);

		if (OS_ERR_NONE != os_err)
		{
			while (1);
		}
	}
	CPU_CRITICAL_EXIT();

	OSStart(&os_err);
	while (1);

	return 0;
}
