
/*
 * ******************************************************************************************
 * File		: csr.h
 * Author	: GowinSemicoductor
 * Chip		: AE350_SOC
 * Function	: Control and status register (CSR) definitions
 * ******************************************************************************************
 */

#ifndef __CSR_V5_H__
#define __CSR_V5_H__


// Definitions -------------------------------------------------------------------------------

#define  NDS_USTATUS          0x000
#define  NDS_UIE              0x004
#define  NDS_UTVEC            0x005
#define  NDS_USCRATCH         0x040
#define  NDS_UEPC             0x041
#define  NDS_UCAUSE           0x042
#define  NDS_UTVAL            0x043
#define  NDS_UIP              0x044
#define  NDS_FFLAGS           0x001
#define  NDS_FRM              0x002
#define  NDS_FCSR             0x003
#define  NDS_CYCLE            0xc00
#define  NDS_TIME             0xc01
#define  NDS_INSTRET          0xc02
#define  NDS_HPMCOUNTER3      0xc03
#define  NDS_HPMCOUNTER4      0xc04
#define  NDS_HPMCOUNTER5      0xc05
#define  NDS_HPMCOUNTER6      0xc06
#define  NDS_HPMCOUNTER7      0xc07
#define  NDS_HPMCOUNTER8      0xc08
#define  NDS_HPMCOUNTER9      0xc09
#define  NDS_HPMCOUNTER10     0xc0a
#define  NDS_HPMCOUNTER11     0xc0b
#define  NDS_HPMCOUNTER12     0xc0c
#define  NDS_HPMCOUNTER13     0xc0d
#define  NDS_HPMCOUNTER14     0xc0e
#define  NDS_HPMCOUNTER15     0xc0f
#define  NDS_HPMCOUNTER16     0xc10
#define  NDS_HPMCOUNTER17     0xc11
#define  NDS_HPMCOUNTER18     0xc12
#define  NDS_HPMCOUNTER19     0xc13
#define  NDS_HPMCOUNTER20     0xc14
#define  NDS_HPMCOUNTER21     0xc15
#define  NDS_HPMCOUNTER22     0xc16
#define  NDS_HPMCOUNTER23     0xc17
#define  NDS_HPMCOUNTER24     0xc18
#define  NDS_HPMCOUNTER25     0xc19
#define  NDS_HPMCOUNTER26     0xc1a
#define  NDS_HPMCOUNTER27     0xc1b
#define  NDS_HPMCOUNTER28     0xc1c
#define  NDS_HPMCOUNTER29     0xc1d
#define  NDS_HPMCOUNTER30     0xc1e
#define  NDS_HPMCOUNTER31     0xc1f
#define  NDS_CYCLEH           0xc80
#define  NDS_TIMEH            0xc81
#define  NDS_INSTRETH         0xc82
#define  NDS_HPMCOUNTER3H     0xc83
#define  NDS_HPMCOUNTER4H     0xc84
#define  NDS_HPMCOUNTER5H     0xc85
#define  NDS_HPMCOUNTER6H     0xc86
#define  NDS_HPMCOUNTER7H     0xc87
#define  NDS_HPMCOUNTER8H     0xc88
#define  NDS_HPMCOUNTER9H     0xc89
#define  NDS_HPMCOUNTER10H    0xc8a
#define  NDS_HPMCOUNTER11H    0xc8b
#define  NDS_HPMCOUNTER12H    0xc8c
#define  NDS_HPMCOUNTER13H    0xc8d
#define  NDS_HPMCOUNTER14H    0xc8e
#define  NDS_HPMCOUNTER15H    0xc8f
#define  NDS_HPMCOUNTER16H    0xc90
#define  NDS_HPMCOUNTER17H    0xc91
#define  NDS_HPMCOUNTER18H    0xc92
#define  NDS_HPMCOUNTER19H    0xc93
#define  NDS_HPMCOUNTER20H    0xc94
#define  NDS_HPMCOUNTER21H    0xc95
#define  NDS_HPMCOUNTER22H    0xc96
#define  NDS_HPMCOUNTER23H    0xc97
#define  NDS_HPMCOUNTER24H    0xc98
#define  NDS_HPMCOUNTER25H    0xc99
#define  NDS_HPMCOUNTER26H    0xc9a
#define  NDS_HPMCOUNTER27H    0xc9b
#define  NDS_HPMCOUNTER28H    0xc9c
#define  NDS_HPMCOUNTER29H    0xc9d
#define  NDS_HPMCOUNTER30H    0xc9e
#define  NDS_HPMCOUNTER31H    0xc9f
#define  NDS_SSTATUS          0x100
#define  NDS_SEDELEG          0x102
#define  NDS_SIDELEG          0x103
#define  NDS_SIE              0x104
#define  NDS_STVEC            0x105
#define  NDS_SCOUNTEREN       0x106
#define  NDS_SSCRATCH         0x140
#define  NDS_SEPC             0x141
#define  NDS_SCAUSE           0x142
#define  NDS_STVAL            0x143
#define  NDS_SIP              0x144
#define  NDS_SATP             0x180
#define  NDS_MVENDORID        0xf11
#define  NDS_MARCHID          0xf12
#define  NDS_MIMPID           0xf13
#define  NDS_MHARTID          0xf14
#define  NDS_MSTATUS          0x300
#define  NDS_MISA             0x301
#define  NDS_MEDELEG          0x302
#define  NDS_MIDELEG          0x303
#define  NDS_MIE              0x304
#define  NDS_MTVEC            0x305
#define  NDS_MCOUNTEREN       0x306
#define  NDS_MSCRATCH         0x340
#define  NDS_MEPC             0x341
#define  NDS_MCAUSE           0x342
#define  NDS_MTVAL            0x343
#define  NDS_MIP              0x344
#define  NDS_PMPCFG0          0x3a0
#define  NDS_PMPCFG1          0x3a1
#define  NDS_PMPCFG2          0x3a2
#define  NDS_PMPCFG3          0x3a3
#define  NDS_PMPADDR0         0x3b0
#define  NDS_PMPADDR1         0x3b1
#define  NDS_PMPADDR2         0x3b2
#define  NDS_PMPADDR3         0x3b3
#define  NDS_PMPADDR4         0x3b4
#define  NDS_PMPADDR5         0x3b5
#define  NDS_PMPADDR6         0x3b6
#define  NDS_PMPADDR7         0x3b7
#define  NDS_PMPADDR8         0x3b8
#define  NDS_PMPADDR9         0x3b9
#define  NDS_PMPADDR10        0x3ba
#define  NDS_PMPADDR11        0x3bb
#define  NDS_PMPADDR12        0x3bc
#define  NDS_PMPADDR13        0x3bd
#define  NDS_PMPADDR14        0x3be
#define  NDS_PMPADDR15        0x3bf
#define  NDS_MCYCLE           0xb00
#define  NDS_MINSTRET         0xb02
#define  NDS_MHPMCOUNTER3     0xb03
#define  NDS_MHPMCOUNTER4     0xb04
#define  NDS_MHPMCOUNTER5     0xb05
#define  NDS_MHPMCOUNTER6     0xb06
#define  NDS_MHPMCOUNTER7     0xb07
#define  NDS_MHPMCOUNTER8     0xb08
#define  NDS_MHPMCOUNTER9     0xb09
#define  NDS_MHPMCOUNTER10    0xb0a
#define  NDS_MHPMCOUNTER11    0xb0b
#define  NDS_MHPMCOUNTER12    0xb0c
#define  NDS_MHPMCOUNTER13    0xb0d
#define  NDS_MHPMCOUNTER14    0xb0e
#define  NDS_MHPMCOUNTER15    0xb0f
#define  NDS_MHPMCOUNTER16    0xb10
#define  NDS_MHPMCOUNTER17    0xb11
#define  NDS_MHPMCOUNTER18    0xb12
#define  NDS_MHPMCOUNTER19    0xb13
#define  NDS_MHPMCOUNTER20    0xb14
#define  NDS_MHPMCOUNTER21    0xb15
#define  NDS_MHPMCOUNTER22    0xb16
#define  NDS_MHPMCOUNTER23    0xb17
#define  NDS_MHPMCOUNTER24    0xb18
#define  NDS_MHPMCOUNTER25    0xb19
#define  NDS_MHPMCOUNTER26    0xb1a
#define  NDS_MHPMCOUNTER27    0xb1b
#define  NDS_MHPMCOUNTER28    0xb1c
#define  NDS_MHPMCOUNTER29    0xb1d
#define  NDS_MHPMCOUNTER30    0xb1e
#define  NDS_MHPMCOUNTER31    0xb1f
#define  NDS_MCYCLEH          0xb80
#define  NDS_MINSTRETH        0xb82
#define  NDS_MHPMCOUNTER3H    0xb83
#define  NDS_MHPMCOUNTER4H    0xb84
#define  NDS_MHPMCOUNTER5H    0xb85
#define  NDS_MHPMCOUNTER6H    0xb86
#define  NDS_MHPMCOUNTER7H    0xb87
#define  NDS_MHPMCOUNTER8H    0xb88
#define  NDS_MHPMCOUNTER9H    0xb89
#define  NDS_MHPMCOUNTER10H   0xb8a
#define  NDS_MHPMCOUNTER11H   0xb8b
#define  NDS_MHPMCOUNTER12H   0xb8c
#define  NDS_MHPMCOUNTER13H   0xb8d
#define  NDS_MHPMCOUNTER14H   0xb8e
#define  NDS_MHPMCOUNTER15H   0xb8f
#define  NDS_MHPMCOUNTER16H   0xb90
#define  NDS_MHPMCOUNTER17H   0xb91
#define  NDS_MHPMCOUNTER18H   0xb92
#define  NDS_MHPMCOUNTER19H   0xb93
#define  NDS_MHPMCOUNTER20H   0xb94
#define  NDS_MHPMCOUNTER21H   0xb95
#define  NDS_MHPMCOUNTER22H   0xb96
#define  NDS_MHPMCOUNTER23H   0xb97
#define  NDS_MHPMCOUNTER24H   0xb98
#define  NDS_MHPMCOUNTER25H   0xb99
#define  NDS_MHPMCOUNTER26H   0xb9a
#define  NDS_MHPMCOUNTER27H   0xb9b
#define  NDS_MHPMCOUNTER28H   0xb9c
#define  NDS_MHPMCOUNTER29H   0xb9d
#define  NDS_MHPMCOUNTER30H   0xb9e
#define  NDS_MHPMCOUNTER31H   0xb9f
#define  NDS_MHPMEVENT3       0x323
#define  NDS_MHPMEVENT4       0x324
#define  NDS_MHPMEVENT5       0x325
#define  NDS_MHPMEVENT6       0x326
#define  NDS_MHPMEVENT7       0x327
#define  NDS_MHPMEVENT8       0x328
#define  NDS_MHPMEVENT9       0x329
#define  NDS_MHPMEVENT10      0x32a
#define  NDS_MHPMEVENT11      0x32b
#define  NDS_MHPMEVENT12      0x32c
#define  NDS_MHPMEVENT13      0x32d
#define  NDS_MHPMEVENT14      0x32e
#define  NDS_MHPMEVENT15      0x32f
#define  NDS_MHPMEVENT16      0x330
#define  NDS_MHPMEVENT17      0x331
#define  NDS_MHPMEVENT18      0x332
#define  NDS_MHPMEVENT19      0x333
#define  NDS_MHPMEVENT20      0x334
#define  NDS_MHPMEVENT21      0x335
#define  NDS_MHPMEVENT22      0x336
#define  NDS_MHPMEVENT23      0x337
#define  NDS_MHPMEVENT24      0x338
#define  NDS_MHPMEVENT25      0x339
#define  NDS_MHPMEVENT26      0x33a
#define  NDS_MHPMEVENT27      0x33b
#define  NDS_MHPMEVENT28      0x33c
#define  NDS_MHPMEVENT29      0x33d
#define  NDS_MHPMEVENT30      0x33e
#define  NDS_MHPMEVENT31      0x33f
#define  NDS_TSELECT          0x7a0
#define  NDS_TDATA1           0x7a1
#define  NDS_TDATA2           0x7a2
#define  NDS_TDATA3           0x7a3
#define  NDS_TINFO            0x7a4
#define  NDS_DCSR             0x7b0
#define  NDS_DPC              0x7b1
#define  NDS_HSTATUS          0x200
#define  NDS_HEDELEG          0x202
#define  NDS_HIDELEG          0x203
#define  NDS_HIE              0x204
#define  NDS_HTVEC            0x205
#define  NDS_HSCRATCH         0x240
#define  NDS_HEPC             0x241
#define  NDS_HCAUSE           0x242
#define  NDS_HBADADDR         0x243
#define  NDS_HIP              0x244
#define  NDS_MBASE            0x380
#define  NDS_MBOUND           0x381
#define  NDS_MIBASE           0x382
#define  NDS_MIBOUND          0x383
#define  NDS_MDBASE           0x384
#define  NDS_MDBOUND          0x385
#define  NDS_MUCOUNTEREN      0x320
#define  NDS_MSCOUNTEREN      0x321
#define  NDS_MHCOUNTEREN      0x322
#define  NDS_UITB             0x800
#define  NDS_DSCRATCH0        0x7b2
#define  NDS_DSCRATCH1        0x7b3
#define  NDS_MICM_CFG         0xfc0
#define  NDS_MDCM_CFG         0xfc1
#define  NDS_MMSC_CFG         0xfc2
#define  NDS_MILMB            0x7c0
#define  NDS_MDLMB            0x7c1
#define  NDS_MECC_CODE        0x7c2
#define  NDS_MNVEC            0x7c3
#define  NDS_MXSTATUS         0x7c4
#define  NDS_MPFT_CTL         0x7c5
#define  NDS_MHSP_CTL         0x7c6
#define  NDS_MSP_BOUND        0x7c7
#define  NDS_MSP_BASE         0x7c8
#define  NDS_MDCAUSE          0x7c9
#define  NDS_MCACHE_CTL       0x7ca
#define  NDS_MMISC_CTL        0x7d0
#define  NDS_DEXC2DBG         0x7e0
#define  NDS_DDCAUSE          0x7e1
#define  NDS_MRANDSEQ         0x7fc
#define  NDS_MRANDSEQH        0x7fd
#define  NDS_MRANDSTATE       0x7fe
#define  NDS_MRANDSTATEH      0x7ff
#define  NDS_MCOUNTERWEN      0x7ce
#define  NDS_MCOUNTERINTEN    0x7cf
#define  NDS_MCOUNTERMASK_M   0x7d1
#define  NDS_MCOUNTERMASK_S   0x7d2
#define  NDS_MCOUNTERMASK_U   0x7d3
#define  NDS_MCOUNTEROVF      0x7d4
#define  NDS_MSLIDELEG        0x7d5
#define  NDS_MCCTLBEGINADDR   0x7cb
#define  NDS_MCCTLCOMMAND     0x7cc
#define  NDS_MCCTLDATA        0x7cd
#define  NDS_UCCTLBEGINADDR   0x80b
#define  NDS_UCCTLCOMMAND     0x80c
#define  NDS_SLIE             0x9c4
#define  NDS_SLIP             0x9c5
#define  NDS_SDCAUSE          0x9c9
#define  NDS_SCCTLDATA        0x9cd
#define  NDS_SCOUNTERINTEN    0x9cf
#define  NDS_SCOUNTERMASK_S   0x9d2
#define  NDS_SCOUNTERMASK_U   0x9d3
#define  NDS_SCOUNTEROVF      0x9d4
#define  NDS_SMISC_CTL        0x9d0
#define  NDS_SCOUNTERMASK_M   0x9d1
#define  NDS_SHPMEVENT3       0x9e3
#define  NDS_SHPMEVENT4       0x9e4
#define  NDS_SHPMEVENT5       0x9e5
#define  NDS_SHPMEVENT6       0x9e6
#define  NDS_MSAVESTATUS      0x7d6
#define  NDS_MSAVEEPC1        0x7d7
#define  NDS_MSAVECAUSE1      0x7d8
#define  NDS_MSAVEEPC2        0x7d9
#define  NDS_MSAVECAUSE2      0x7da
#define  NDS_SCOUNTINHIBIT    0x9e0
#define  NDS_UDCAUSE          0x809
#define  NDS_WFE              0x810
#define  NDS_SLEEPVALUE       0x811
#define  NDS_TXEVT            0x812
#define  NDS_MPPIB            0x7f0
#define  NDS_MFIOB            0x7f1
#define  NDS_MTVT             0x307
#define  NDS_MNXTI            0x345
#define  NDS_MINTSTATUS       0x346
#define  NDS_MSCRATCHCSW      0x348
#define  NDS_MSCRATCHCSWL     0x349
#define  NDS_MSAVEDCAUSE1     0x7db
#define  NDS_MSAVEDCAUSE2     0x7dc
#define  NDS_PUSHMXSTATUS     0x7eb
#define  NDS_MIRQ_ENTRY       0x7ec
#define  NDS_MINTSEL_JAL      0x7ed
#define  NDS_PUSHMCAUSE       0x7ee
#define  NDS_PUSHMEPC         0x7ef
#define  NDS_UCODE            0x801
#define  NDS_PMACFG0          0xbc0
#define  NDS_PMACFG1          0xbc1
#define  NDS_PMACFG2          0xbc2
#define  NDS_PMACFG3          0xbc3
#define  NDS_PMAADDR0         0xbd0
#define  NDS_PMAADDR1         0xbd1
#define  NDS_PMAADDR2         0xbd2
#define  NDS_PMAADDR3         0xbd3
#define  NDS_PMAADDR4         0xbd4
#define  NDS_PMAADDR5         0xbd5
#define  NDS_PMAADDR6         0xbd6
#define  NDS_PMAADDR7         0xbd7
#define  NDS_PMAADDR8         0xbd8
#define  NDS_PMAADDR9         0xbd9
#define  NDS_PMAADDR10        0xbda
#define  NDS_PMAADDR11        0xbdb
#define  NDS_PMAADDR12        0xbdc
#define  NDS_PMAADDR13        0xbdd
#define  NDS_PMAADDR14        0xbde
#define  NDS_PMAADDR15        0xbdf
#define  NDS_MMSC_CFG2        0xfc3


#endif	/* __CSR_V5_H__ */
